﻿/*
 Copyright (c) 2013, Earth People Technology. All rights reserved.
 http://www.earthpeopletechnology.com

 Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission 
 notice appear in all copies.

 THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. 
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
 WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Text.RegularExpressions;
using Microsoft.Win32;


namespace EPT_Transfer_Test
{
    public partial class EPT_Transfer_Test : Form
    {

        public EPT_Transfer_Test()
        {
            InitializeComponent();

            // Marks the device as closed in the GUI
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;
            lblDeviceConnected.Text = "Dis-Connected";

            btnLEDReset.Enabled = false;
            btnWriteByte.Enabled = false;
            tbWriteMultiByte.Enabled = false;
            btnWriteMultiByte.Enabled = false;
            btnTransferReset.Enabled = false;
            btnResetTriggerOut.Enabled = false;
            btnBlinky.Enabled = false;
            btnSlideSwRead.Enabled = false;
            btnGPIOReadAll.Enabled = false;
            btnLEDReset.Enabled = false;
            btnLED1Green.Enabled = false;
            btnLED1Red.Enabled = false;
            btnLED1Blue.Enabled = false;
            btnLED2Green.Enabled = false;
            btnLED2Red.Enabled = false;
            btnLED2Blue.Enabled = false;
            btnLED3Green.Enabled = false;
            btnLED3Red.Enabled = false;
            btnLED3Blue.Enabled = false;
            btnLED4Green.Enabled = false;
            btnLED4Red.Enabled = false;
            btnLED4Blue.Enabled = false;

            btnGPIO_LB_IOH_7.Enabled = false;
            btnGPIO_LB_IOH_6.Enabled = false;
            btnGPIO_LB_IOH_5.Enabled = false;
            btnGPIO_LB_IOH_4.Enabled = false;
            btnGPIO_LB_IOH_3.Enabled = false;
            btnGPIO_LB_IOH_2.Enabled = false;
            btnGPIO_LB_IOH_1.Enabled = false;
            btnGPIO_LB_IOH_0.Enabled = false;

            btnGPIO_LB_IOL_7.Enabled = false;
            btnGPIO_LB_IOL_6.Enabled = false;
            btnGPIO_LB_IOL_5.Enabled = false;
            btnGPIO_LB_IOL_4.Enabled = false;
            btnGPIO_LB_IOL_3.Enabled = false;
            btnGPIO_LB_IOL_2.Enabled = false;
            btnGPIO_LB_IOL_1.Enabled = false;
            btnGPIO_LB_IOL_0.Enabled = false;

            btnGPIO_LB_AD_5.Enabled = false;
            btnGPIO_LB_AD_4.Enabled = false;
            btnGPIO_LB_AD_3.Enabled = false;
            btnGPIO_LB_AD_2.Enabled = false;
            btnGPIO_LB_AD_1.Enabled = false;
            btnGPIO_LB_AD_0.Enabled = false;

            btnGPIO_LB_SER_1.Enabled = false;
            btnGPIO_LB_SER_0.Enabled = false;

            btnSlideSwitch1.Enabled = false;
            btnSlideSwitch2.Enabled = false;

            btnGPIOLegendUp.Enabled = false;
            btnGPIOLegendLo.Enabled = false;

            for (int i = 0; i < EPTTransmitDevice.Length; ++i)
            {
                EPTTransmitDevice[i] = new Transfer();
            }

            //rbtnRepitions.Checked = true;
            //rbtnInfinite.Checked = false;
        }

        //Sting array to hold the Port names that match the VID = "403 PID = "6010"
        public string[] MatchingComPortList = new string[10];

        // Local Serial Port parameters From Properties Page
        public static string PortName = "COM4";
        public static string BaudRate = "115200";
        public static string DataBits = "8";
        public static string vParity = "None";
        public static string StopBits = "1";
        public static string pHandshake = "None";

        // Serial port send variables
        byte[] PutBuff;
        //char[] PutChar;

        //Flag  for converting incoming stream to Hex
        public static bool HexConversion = false;
        public static int IdxHexChar = 0;
        public static bool hexConversionTabFlag = false;

        //Line Feed
        public static bool EchoTransmitCharacters = true;
        public static bool AppendLineFeed = false;

        //EPT Receive Command Parameters
        public const byte TRIGGER_IN_COMMAND = 0x08;
        public const byte TRANSFER_IN_COMMAND = 0x10;
        public const byte BLOCK_IN_COMMAND = 0x20;

        // EPT EndTerm parameters
        public const byte TRANSFER_ENDTERM_1 = 0x01;
        public const byte TRANSFER_ENDTERM_2 = 0x02;
        public const byte TRANSFER_ENDTERM_3 = 0x03;
        public const byte TRANSFER_ENDTERM_4 = 0x04;
        public const byte TRANSFER_ENDTERM_5 = 0x05;
        public const byte TRANSFER_ENDTERM_6 = 0x06;
        public const byte TRANSFER_ENDTERM_7 = 0x07;

        //Parameters for EPT Receive object Trigger
        public const byte TRIGGER_IN_BYTE_1 = 0x01;
        public const byte TRIGGER_IN_BYTE_2 = 0x02;
        public const byte TRIGGER_IN_BYTE_4 = 0x04;
        public const byte TRIGGER_IN_BYTE_8 = 0x08;
        public const byte TRIGGER_IN_BYTE_10 = 0x10;
        public const byte TRIGGER_IN_BYTE_20 = 0x20;
        public const byte TRIGGER_IN_BYTE_40 = 0x40;
        public const byte TRIGGER_IN_BYTE_80 = 0x80;

        //Parameters for EPT Receive object Transfer byte
        public const byte TRANSFER_IN_ADDRESS_1 = 0x1;
        public const byte TRANSFER_IN_ADDRESS_2 = 0x2;

        //Parameters for EPT Receive object Block
        public const byte BLOCK_IN_ADDRESS_1 = 0x1;
        public const byte BLOCK_IN_ADDRESS_2 = 0x2;

        //Transfer Parameters
        public const byte CONTROL_REG_RESET = 0x00;
        public const byte TRANSFER_LOOPBACK = 0x01;
        public const byte BLOCK_LOOPBACK = 0x02;
        public const byte LOAD_IMAGE_TO_BUFFER = 0x04;
        public const byte LOAD_FACE_IMAGE = 0x08;
        public const byte RANDOM_IMAGE = 0x10;
        public const byte SET_SHIFT_COUNT_UP = 0x40;
        public const byte SET_SHIFT_COUNT_DOWN = 0x50;
        public const byte SET_STATIC_IMAGE = 0x70;
        public const byte SET_SHIFT_RIGHT_IMAGE = 0x90;
        public const byte SET_SHIFT_LEFT_IMAGE = 0xa0;

        //CPL Memory registers to send to device
        readonly byte TRIGGER_WRITE_LATCH = 0x01;
        readonly byte TRIGGER_OE_LATCH = 0x02;
        readonly byte TRIGGER_READ_LATCH = 0x04;

        //Create a Receive object of the Transfer Class.
        //Transfer EPTReceiveDevice = new Transfer();

        //Create an array of the Transfer Class for device
        Transfer[] EPTTransmitDevice = new Transfer[8];

        //String to Hold payload
        public String PayloadString = "";

        //Hold the previous status of the LED
        //int LEDStatusPrevious = 0;

        //LED Status Register
        //byte LEDStatus;

        //Parameters
        public const int SELECT_RANDOM = 1;
        public const int SELECT_SHIFT_LEFT = 2;
        public const int SELECT_SHIFT_RIGHT = 3;
        public const int SELECT_COUNT_UP = 4;
        public const int SELECT_COUNT_DOWN = 5;

        //Parameters
        public const byte userLEDAddress = 0x01;
        public const byte userByteAddress = 0x02;
        //Create an array of the Transfer Class for device
/*        Transfer[] EPTTransmitDevice = new Transfer[8];
*/
        //Block Transfer and Loop counter
        uint BlockCount = 0;
        int LoopCount = 0;

        //Block Transfer Control 
        public bool BlockTransferInfinite = false;
        public bool BlockTransferReps = false;
        public bool BlockTransferStop = false;

        //LED Blinky global registers
        bool LEDBlinkyFlag = false;
        byte ledByteAddress = 0x03;
        public bool led1GreenEnable = true;
        public bool led1RedEnable = true;
        public bool led1BlueEnable = true;

        public bool led2GreenEnable = true;
        public bool led2RedEnable = true;
        public bool led2BlueEnable = true;

        public bool led3GreenEnable = true;
        public bool led3RedEnable = true;
        public bool led3BlueEnable = true;

        public bool led4GreenEnable = true;
        public bool led4RedEnable = true;
        public bool led4BlueEnable = true;

        //GPIO Registers
        public byte gpioLB_IOL;
        public byte gpioLB_IOH;
        public byte gpioLB_AD;
        public byte gpioLB_SER;
        public bool gpioInfiniteLoopFlag = false;
        public bool gpioSingleLoopFlag = true;
        public bool gpioInfiniteLoopEnabled = false;


        // Main object loader
        private void EPT_Transfer_Test_Load(object sender, System.EventArgs e)
        {
            //String buffer
            String PortText = "";

            //Index registers
            int Index = 0, EPTgroupNumber = 0;

            // Call the List Devices function
            List<string> names = ComPortNames("0403", "6010");

            // Get a list of serial port names.
            string[] ports;
            ports = SerialPort.GetPortNames();


            if (names.Count > 0)
            {
                foreach (String port in ports)
                {
                    //Compare port name with the found VID/PID 
                    //combinations. Add them to Matching port list
                    //and comboDevList
                    if (names.Contains(port))
                    {
                        MatchingComPortList[Index] = port;

                        if (Index == 0)
                        {
                            PortText = "EPT JTAG Blaster " + EPTgroupNumber;
                            Index++;
                        }
                        else
                        {
                            PortText = "EPT Serial Communications " + EPTgroupNumber++;
                            Index++;
                        }
                        cmbDevList.Items.Add(PortText);
                    }
                }
            }
            else
                MessageBox.Show("No EPT Devices found!");


            //SetButtonEnables_Close();

        }


        private void SetButtonEnables()
        {
            string s2, s3;

            // Marks the device as closed in the GUI
            btnOpenDevice.Enabled = false;
            btnCloseDevice.Enabled = true;


            btnLEDReset.Enabled = true;
            btnWriteByte.Enabled = true;
            tbWriteMultiByte.Enabled = true;
            btnWriteMultiByte.Enabled = true;
            btnTransferReset.Enabled = true;
            btnResetTriggerOut.Enabled = true;
            btnBlinky.Enabled = true;
            btnSlideSwRead.Enabled = true;
            btnGPIOReadAll.Enabled = true;
            btnLEDReset.Enabled = true;
            btnLED1Green.Enabled = true;
            btnLED1Red.Enabled = true;
            btnLED1Blue.Enabled = true;
            btnLED2Green.Enabled = true;
            btnLED2Red.Enabled = true;
            btnLED2Blue.Enabled = true;
            btnLED3Green.Enabled = true;
            btnLED3Red.Enabled = true;
            btnLED3Blue.Enabled = true;
            btnLED4Green.Enabled = true;
            btnLED4Red.Enabled = true;
            btnLED4Blue.Enabled = true;

            btnGPIO_LB_IOH_7.Enabled = false;
            btnGPIO_LB_IOH_6.Enabled = false;
            btnGPIO_LB_IOH_5.Enabled = false;
            btnGPIO_LB_IOH_4.Enabled = false;
            btnGPIO_LB_IOH_3.Enabled = false;
            btnGPIO_LB_IOH_2.Enabled = false;
            btnGPIO_LB_IOH_1.Enabled = false;
            btnGPIO_LB_IOH_0.Enabled = false;

            btnGPIO_LB_IOL_7.Enabled = false;
            btnGPIO_LB_IOL_6.Enabled = false;
            btnGPIO_LB_IOL_5.Enabled = false;
            btnGPIO_LB_IOL_4.Enabled = false;
            btnGPIO_LB_IOL_3.Enabled = false;
            btnGPIO_LB_IOL_2.Enabled = false;
            btnGPIO_LB_IOL_1.Enabled = false;
            btnGPIO_LB_IOL_0.Enabled = false;

            btnGPIO_LB_AD_5.Enabled = false;
            btnGPIO_LB_AD_4.Enabled = false;
            btnGPIO_LB_AD_3.Enabled = false;
            btnGPIO_LB_AD_2.Enabled = false;
            btnGPIO_LB_AD_1.Enabled = false;
            btnGPIO_LB_AD_0.Enabled = false;

            btnGPIO_LB_SER_1.Enabled = false;
            btnGPIO_LB_SER_0.Enabled = false;

            btnSlideSwitch1.Enabled = false;
            btnSlideSwitch2.Enabled = false;

            btnGPIOLegendUp.Enabled = false;
            btnGPIOLegendLo.Enabled = false;


            s2 = char.ConvertFromUtf32(0x2191);
            btnGPIOLegendLo.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
            btnGPIOLegendLo.BackColor = Color.Red;

            s3 = char.ConvertFromUtf32(0x2193);
            btnGPIOLegendUp.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
            btnGPIOLegendUp.BackColor = Color.DodgerBlue;

            this.Invoke(new MethodInvoker(delegate () { btnGPIOLegendLo.Text = s2; }));
            this.Invoke(new MethodInvoker(delegate () { btnGPIOLegendUp.Text = s3; }));

        }


        private void cmbDevList_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Get Port name from Combo Box
            PortName = MatchingComPortList[cmbDevList.SelectedIndex];
        }

        // Handle "Open" device button
        private void btnOpenDevice_Click(object sender, EventArgs e)
        {
            //Open the Device
            OpenSerialPort1();
            if (serialPort_AH.IsOpen)
            {
                lblDeviceConnected.Text = "Device Connected";
                SetButtonEnables_Open();
            }
        }

        // Handle "Close" device button
        private void btnCloseDevice_Click(object sender, EventArgs e)
        {
            //Set the LED Blinky flag to false to prevent
            //writing to the serial port
            LEDBlinkyFlag = false;

            if (serialPort_AH.IsOpen)
                {
                    CloseSerialPort1();
                    lblDeviceConnected.Text = "Dis-Connected";
                    SetButtonEnables_Close();

                }
        }

        private void SetButtonEnables_Open()
        {

            string s2, s3;

            // Marks the device as closed in the GUI
            btnOpenDevice.Enabled = false;
            btnCloseDevice.Enabled = true;
            lblDeviceConnected.Text = "Device Connected";

            btnLEDReset.Enabled = true;
            btnWriteByte.Enabled = true;
            tbWriteMultiByte.Enabled = true;
            btnWriteMultiByte.Enabled = true;
            btnTransferReset.Enabled = true;
            btnResetTriggerOut.Enabled = true;
            btnGPIOReadAll.Enabled = true;
            btnBlinky.Enabled = true;
            btnSlideSwRead.Enabled = true;
            btnGPIOReadAll.Enabled = true;
            btnLEDReset.Enabled = true;
            btnLED1Green.Enabled = true;
            btnLED1Red.Enabled = true;
            btnLED1Blue.Enabled = true;
            btnLED2Green.Enabled = true;
            btnLED2Red.Enabled = true;
            btnLED2Blue.Enabled = true;
            btnLED3Green.Enabled = true;
            btnLED3Red.Enabled = true;
            btnLED3Blue.Enabled = true;
            btnLED4Green.Enabled = true;
            btnLED4Red.Enabled = true;
            btnLED4Blue.Enabled = true;

            btnGPIO_LB_IOH_7.Enabled = true;
            btnGPIO_LB_IOH_6.Enabled = true;
            btnGPIO_LB_IOH_5.Enabled = true;
            btnGPIO_LB_IOH_4.Enabled = true;
            btnGPIO_LB_IOH_3.Enabled = true;
            btnGPIO_LB_IOH_2.Enabled = true;
            btnGPIO_LB_IOH_1.Enabled = true;
            btnGPIO_LB_IOH_0.Enabled = true;

            btnGPIO_LB_IOL_7.Enabled = true;
            btnGPIO_LB_IOL_6.Enabled = true;
            btnGPIO_LB_IOL_5.Enabled = true;
            btnGPIO_LB_IOL_4.Enabled = true;
            btnGPIO_LB_IOL_3.Enabled = true;
            btnGPIO_LB_IOL_2.Enabled = true;
            btnGPIO_LB_IOL_1.Enabled = true;
            btnGPIO_LB_IOL_0.Enabled = true;

            btnGPIO_LB_AD_5.Enabled = true;
            btnGPIO_LB_AD_4.Enabled = true;
            btnGPIO_LB_AD_3.Enabled = true;
            btnGPIO_LB_AD_2.Enabled = true;
            btnGPIO_LB_AD_1.Enabled = true;
            btnGPIO_LB_AD_0.Enabled = true;

            btnGPIO_LB_SER_1.Enabled = true;
            btnGPIO_LB_SER_0.Enabled = true;

            btnSlideSwitch1.Enabled = true;
            btnSlideSwitch2.Enabled = true;

            btnGPIOLegendUp.Enabled = true;
            btnGPIOLegendLo.Enabled = true;


            // Marks the device as closed in the GUI
            btnOpenDevice.Enabled = false;
            btnCloseDevice.Enabled = true;

            s2 = char.ConvertFromUtf32(0x2191);
            btnGPIOLegendUp.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
            btnGPIOLegendUp.BackColor = Color.Red;

            s3 = char.ConvertFromUtf32(0x2193);
            btnGPIOLegendLo.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
            btnGPIOLegendLo.BackColor = Color.DodgerBlue;

            this.Invoke(new MethodInvoker(delegate () { btnGPIOLegendUp.Text = s2; }));
            this.Invoke(new MethodInvoker(delegate () { btnGPIOLegendLo.Text = s3; }));
        }


        private void SetButtonEnables_Close()
        {
            // Marks the device as closed in the GUI
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;
            lblDeviceConnected.Text = "Dis-Connected";

            btnLEDReset.Enabled = false;
            btnWriteByte.Enabled = false;
            tbWriteMultiByte.Enabled = false;
            btnWriteMultiByte.Enabled = false;
            btnTransferReset.Enabled = false;
            btnResetTriggerOut.Enabled = false;
            LEDBlinkyFlag = false;
            btnBlinky.Enabled = false;
            btnSlideSwRead.Enabled = false;
            btnGPIOReadAll.Enabled = false;
            btnLEDReset.Enabled = false;
            btnLED1Green.Enabled = false;
            btnLED1Red.Enabled = false;
            btnLED1Blue.Enabled = false;
            btnLED2Green.Enabled = false;
            btnLED2Red.Enabled = false;
            btnLED2Blue.Enabled = false;
            btnLED3Green.Enabled = false;
            btnLED3Red.Enabled = false;
            btnLED3Blue.Enabled = false;
            btnLED4Green.Enabled = false;
            btnLED4Red.Enabled = false;
            btnLED4Blue.Enabled = false;

            btnGPIO_LB_IOH_7.Enabled = false;
            btnGPIO_LB_IOH_6.Enabled = false;
            btnGPIO_LB_IOH_5.Enabled = false;
            btnGPIO_LB_IOH_4.Enabled = false;
            btnGPIO_LB_IOH_3.Enabled = false;
            btnGPIO_LB_IOH_2.Enabled = false;
            btnGPIO_LB_IOH_1.Enabled = false;
            btnGPIO_LB_IOH_0.Enabled = false;

            btnGPIO_LB_IOL_7.Enabled = false;
            btnGPIO_LB_IOL_6.Enabled = false;
            btnGPIO_LB_IOL_5.Enabled = false;
            btnGPIO_LB_IOL_4.Enabled = false;
            btnGPIO_LB_IOL_3.Enabled = false;
            btnGPIO_LB_IOL_2.Enabled = false;
            btnGPIO_LB_IOL_1.Enabled = false;
            btnGPIO_LB_IOL_0.Enabled = false;

            btnGPIO_LB_AD_5.Enabled = false;
            btnGPIO_LB_AD_4.Enabled = false;
            btnGPIO_LB_AD_3.Enabled = false;
            btnGPIO_LB_AD_2.Enabled = false;
            btnGPIO_LB_AD_1.Enabled = false;
            btnGPIO_LB_AD_0.Enabled = false;

            btnGPIO_LB_SER_1.Enabled = false;
            btnGPIO_LB_SER_0.Enabled = false;

            btnSlideSwitch1.Enabled = false;
            btnSlideSwitch2.Enabled = false;

            btnGPIOLegendUp.Enabled = false;
            btnGPIOLegendLo.Enabled = false;


        }

        // Handle the "Byte" button
        private void btnWriteByte_Click(object sender, EventArgs e)
        {
            int address_to_device;
            int ibyte = Convert.ToInt32(tbNumBytes.Text);
            address_to_device = Convert.ToInt32(tbAddress.Text);

            EPT_AH_SendByte(userByteAddress, (byte)ibyte);
        }

        // Handle the "Multi Byte" button
        private void btnWriteMultiByte_Click(object sender, EventArgs e)
        {
            int i, address_to_device;
            byte dataToSend;
            address_to_device = Convert.ToInt32(tbAddress.Text);
            string[] result = tbWriteMultiByte.Text.Split(' ');
            for (i = 0; i < result.Length; i++)
            {
                dataToSend = (byte)Convert.ToInt32(result[i]);

            Thread t = new Thread(new ParameterizedThreadStart(multiByteSend));
                t.Start( dataToSend);
                Thread.Sleep(100);
            }

        }

        private void multiByteSend(object data)
        {
            EPT_AH_SendByte((byte)userByteAddress, (byte)data);
        }

        // Handle the "LoopBack" button
        private void btnLoopBack_Click(object sender, EventArgs e)
        {
            EPT_AH_SendTransferControlByte(userByteAddress, (byte)1);
        }


        // Handle the LED trigger "Rst" button
        private void btnLEDReset_Click(object sender, EventArgs e)
        {
            EPT_AH_SendByte((byte)ledByteAddress, (byte)0);

            Thread.Sleep(10);
            EPT_AH_SendTrigger((byte)1);

            Thread.Sleep(10);
            EPT_AH_SendTrigger((byte)2);

            Thread.Sleep(10);
            EPT_AH_SendTrigger((byte)4);

            Thread.Sleep(10);
            EPT_AH_SendTrigger((byte)8);

        }

        // Handle the "Stop" block transfer button
/*        private void btnStopBlockTransfer_Click(object sender, EventArgs e)
        {
            BlockTransferStop = true;
        }
*/
        // Handle the block transfer "Rst" button
        private void btnResetTriggerOut_Click(object sender, EventArgs e)
        {
            EPT_AH_SendTransferControlByte(userByteAddress, (byte)8);
            Thread.Sleep(1);
            EPT_AH_SendTransferControlByte(userByteAddress, (byte)0);
            Thread.Sleep(1);

            lLabelSwitch1.Text = "";
            lLabelSwitch2.Text = "";
        }

        // Handle the "OK" button
        private void btnOk_Click(object sender, EventArgs e)
        {
            if (serialPort_AH.IsOpen)
            {
                CloseSerialPort1();
                lblDeviceConnected.Text = "Dis-Connected";
                SetButtonEnables_Close();

            }

            Application.Exit();
        }

        // Handle the "Cancel" button
        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (serialPort_AH.IsOpen)
            {
                CloseSerialPort1();
                lblDeviceConnected.Text = "Dis-Connected";
                SetButtonEnables_Close();

            }

            Application.Exit();

        }

        // Handle the "Blinky!" lightshow button
        private void btnBlinky_Click(object sender, EventArgs e)
        {
            Int32 blinkTime = 1;

            if (LEDBlinkyFlag)
                LEDBlinkyFlag = false;
            else
            {
                LEDBlinkyFlag = true;

                btnWriteByte.Enabled = false;
                //btnTrigger1.Enabled = false;
                //btnTrigger2.Enabled = false;
                //btnTrigger3.Enabled = false;
                //btnTrigger4.Enabled = false;
                btnLEDReset.Enabled = false;
                //btnBlkCompare4.Enabled = false;
                //btnStopBlockTransfer.Enabled = false;
                //btnResetBlock.Enabled = false;
                btnResetTriggerOut.Enabled = false;
                tbWriteMultiByte.Enabled = false;
                btnWriteMultiByte.Enabled = false;
                btnTransferReset.Enabled = false;
                //btnBlockSendRst.Enabled = false;
                //tbBlockSend.Enabled = false;
                //btnUserBlk.Enabled = false;

                Thread t = new Thread(new ParameterizedThreadStart(LEDBlinky));
                t.Start(blinkTime);

            }

        }

        private void LEDBlinky(object data)
        {
            Int32 blink_time = 100;      // 100 milliseconds
            Int32 sequence_cycles = 100; // Cycle, 100 times
            Int32 sequence_size = 24;    // Size of the LED lightshow sequence


            // DEBUG:
            // Use a random timer instead of 100ms
            Random random = new Random();
            // Go between 50 and 300 milliseconds
            blink_time = random.Next(50, 500);
            sequence_cycles = random.Next(10, 100);

            // Set up light controls according to the following legend:
            // 1: Light 1 ON
            // 2: Light 2 ON
            // 3: Light 3 ON
            // 4: Light 4 ON
            // 0: All lights OFF
            // > 4: Random light ON
            byte[] light_controls = new byte[24] {1, 0, 2, 0, 3, 0, 4, 0, 
                                                       3, 0, 2, 0, 1, 0, 5, 0, 
                                                       5, 0, 5, 0, 1, 4, 2, 3};

            //Call the LED Blinky Function
            EPT_AH_LEDBlinky(blink_time, sequence_cycles, sequence_size);
            //}

        }

        // Parse incoming EPT data
        private void EPTParseReceive()
        {
            // Parse the command
            switch (EPTReceiveDevice.Command)
            {
                // This is a trigger command
                case TRIGGER_IN_COMMAND:
                    TriggerInReceive();
                    break;
                // This is a single-byte transfer command
                case TRANSFER_IN_COMMAND:
                    TransferInReceive();
                    break;
                // This is a data block transfer command
                case BLOCK_IN_COMMAND:
                    BlockInReceive();
                    break;
                default:
                    break;
            }
        }

        // Handle GUI update of an incoming trigger
        public void TriggerInReceive()
        {
            switch (EPTReceiveDevice.Payload)
            {
                case 0x01:
                    lLabelSwitch1.Text = "Switch 1\n Pressed";
                    break;
                case 0x02:
                    lLabelSwitch2.Text = "Switch 2\n Pressed";
                    break;
                case 0x04:
                    lLabelSwitch1.Text = "";
                    lLabelSwitch2.Text = "";
                    break;
            }
        }

        // Handle GUI update of an incoming byte transfer
        public void TransferInReceive()
        {
            switch (EPTReceiveDevice.Address)
            {
                case 0:
                    break;
                case 1:
                    break;
                case 2:
                    loopbackTransferByte();
                    break;
                case 3:
                    slideSwitchSelection();
                    break;
                default: break;
            }

        }

        private void loopbackTransferByte()
        { 
            string WriteRcvChar = "";
            WriteRcvChar = String.Format("{0}", (int)EPTReceiveDevice.Payload);
            this.Invoke(new MethodInvoker(delegate() { tbDataBytes.AppendText(", " + WriteRcvChar); }));
        }

        private void slideSwitchSelection()
        {
            int index = 0;
            int slideSwitchByteIn_0 = 0;
            int slideSwitchByteIn_1 = 0;
            int slideSwitchByteIn_2 = 0;
            int slideSwitchByteIn_3 = 0;
            int slideSwitchByteIn_4 = 0;
            int slideSwitchByteIn_5 = 0;
            int slideSwitchByteIn_6 = 0;
            int slideSwitchByteIn_7 = 0;
            int slideSwitchByteIn = (int)EPTReceiveDevice.Payload;


            for (index = 0; index < 8; index++)
            {
                //gpioLB_XIOLA_0 = ((int)gpioLB_XIOLA & (1 << index));
                switch (index)
                {
                    case 0:
                        slideSwitchByteIn_0 = ((int)slideSwitchByteIn & (1 << index));
                        break;
                    case 1:
                        slideSwitchByteIn_1 = ((int)slideSwitchByteIn & (1 << index));
                        break;
                    case 2:
                        slideSwitchByteIn_2 = ((int)slideSwitchByteIn & (1 << index));
                        break;
                    case 3:
                        slideSwitchByteIn_3 = ((int)slideSwitchByteIn & (1 << index));
                        break;
                    case 4:
                        slideSwitchByteIn_4 = ((int)slideSwitchByteIn & (1 << index));
                        break;
                    case 5:
                        slideSwitchByteIn_5 = ((int)slideSwitchByteIn & (1 << index));
                        break;
                    case 6:
                        slideSwitchByteIn_6 = ((int)slideSwitchByteIn & (1 << index));
                        break;
                    case 7:
                        slideSwitchByteIn_7 = ((int)slideSwitchByteIn & (1 << index));
                        break;
                    default: break;
                }
            }
            //Set the Slide Switch 1 button to the Hi or Lo position
            string s2 = String.Empty;
            if (slideSwitchByteIn_0 == 0x01)
            {
                s2 = char.ConvertFromUtf32(0x2191);
                btnSlideSwitch1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnSlideSwitch1.BackColor = Color.Red;
            }
            else
            {
                s2 = char.ConvertFromUtf32(0x2193);
                btnSlideSwitch1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnSlideSwitch1.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnSlideSwitch1.Text = s2; }));

            //Set the Slide Switch 2 button to the Hi or Lo position
            string s3 = String.Empty;
            if (slideSwitchByteIn_1 == 0x02)
            {
                s3 = char.ConvertFromUtf32(0x2191);
                btnSlideSwitch2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnSlideSwitch2.BackColor = Color.Red;
            }
            else
            {
                s3 = char.ConvertFromUtf32(0x2193);
                btnSlideSwitch2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnSlideSwitch2.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnSlideSwitch2.Text = s3; }));



        }

        // Handle clearing of byte data from the GUI when "Rst" is pressed
        private void btnTransferReset_Click(object sender, EventArgs e)
        {
            tbDataBytes.Clear();
        }

        // Handle GUI update of an incoming block transfer
        public void BlockInReceive()
        {
            BlockCount = BlockCount + EPTReceiveDevice.Length;
            if (EPTReceiveDevice.Address == 4)
            {
                Thread u = new Thread(new ParameterizedThreadStart(DisplayBlockOut));
                u.Start(BlockCount);
            }
        }

        // Display data from the incoming block of transfer
        public void DisplayBlockOut(object CountData)
        {

            //Increment through each byte received and store in the 
            //appropriate GPIO Register
            uint gpioCount = 0;
            foreach (byte b in EPTReceiveDevice.cBlockBuf)
            {

                switch (gpioCount++)
                {
                    case 0: gpioLB_IOL = b;  
                        break;
                    case 1: gpioLB_IOH = b; 
                        break;
                    case 2: gpioLB_AD = b;
                        break;
                    case 3: gpioLB_SER = b;
                        break;
                    default: break;
                }

            }

            //Populate each bit in LB_IOL with the appropriate polarity
            gpioBitSelectionLB_IOL();

            Thread.Sleep(10);

            //Populate each bit in LB_IOH with the appropriate polarity
            gpioBitSelectionLB_IOH();

            Thread.Sleep(10);

            //Populate each bit in gpioLB_AD with the appropriate polarity
            gpioBitSelectionLB_AD();

            //Populate each bit in LB_SER with the appropriate polarity
            gpioBitSelectionLB_SER();

            Thread.Sleep(10);


        }


        //GPIO Bit Polarity Selection for the LB_IOL Connector
        private void gpioBitSelectionLB_IOL()
        {
            int index = 0;
            int gpioLB_IOL_0 = 0;
            int gpioLB_IOL_1 = 0;
            int gpioLB_IOL_2 = 0;
            int gpioLB_IOL_3 = 0;
            int gpioLB_IOL_4 = 0;
            int gpioLB_IOL_5 = 0;
            int gpioLB_IOL_6 = 0;
            int gpioLB_IOL_7 = 0;


            for (index = 0; index < 8; index++)
            {
                //gpioLB_IOL_0 = ((int)gpioLB_IOL & (1 << index));
                switch (index)
                {
                    case 0:
                        gpioLB_IOL_0 = ((int)gpioLB_IOL & (1 << index));
                        break;
                    case 1:
                        gpioLB_IOL_1 = ((int)gpioLB_IOL & (1 << index));
                        break;
                    case 2:
                        gpioLB_IOL_2 = ((int)gpioLB_IOL & (1 << index));
                        break;
                    case 3:
                        gpioLB_IOL_3 = ((int)gpioLB_IOL & (1 << index));
                        break;
                    case 4:
                        gpioLB_IOL_4 = ((int)gpioLB_IOL & (1 << index));
                        break;
                    case 5:
                        gpioLB_IOL_5 = ((int)gpioLB_IOL & (1 << index));
                        break;
                    case 6:
                        gpioLB_IOL_6 = ((int)gpioLB_IOL & (1 << index));
                        break;
                    case 7:
                        gpioLB_IOL_7 = ((int)gpioLB_IOL & (1 << index));
                        break;
                    default: break;
                }
            }
            //Set the LB_IOL Connector Pins to High, Low, Tri
            string s2 = String.Empty;
            if (gpioLB_IOL_0 == 0x01)
            {
                s2 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOL_0.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_0.BackColor = Color.Red;
            }
            else
            {
                s2 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOL_0.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_0.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOL_0.Text = s2; }));

            string s3 = String.Empty;
            if (gpioLB_IOL_1 == 0x02)
            {
                s3 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOL_1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_1.BackColor = Color.Red;
            }
            else
            {
                s3 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOL_1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_1.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOL_1.Text = s3; }));

            string s4 = String.Empty;
            if (gpioLB_IOL_2 == 0x04)
            {
                s4 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOL_2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_2.BackColor = Color.Red;
            }
            else
            {
                s4 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOL_2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_2.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOL_2.Text = s4; }));

            string s5 = String.Empty;
            if (gpioLB_IOL_3 == 0x08)
            {
                s5 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOL_3.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_3.BackColor = Color.Red;
            }
            else
            {
                s5 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOL_3.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_3.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOL_3.Text = s5; }));

            string s6 = String.Empty;
            if (gpioLB_IOL_4 == 0x10)
            {
                s6 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOL_4.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_4.BackColor = Color.Red;
            }
            else
            {
                s6 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOL_4.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_4.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOL_4.Text = s6; }));

            string s7 = String.Empty;
            if (gpioLB_IOL_5 == 0x20)
            {
                s7 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOL_5.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_5.BackColor = Color.Red;
            }
            else
            {
                s7 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOL_5.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_5.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOL_5.Text = s7; }));

            string s8 = String.Empty;
            if (gpioLB_IOL_6 == 0x40)
            {
                s8 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOL_6.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_6.BackColor = Color.Red;
            }
            else
            {
                s8 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOL_6.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_6.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOL_6.Text = s8; }));

            string s9 = String.Empty;
            if (gpioLB_IOL_7 == 0x80)
            {
                s9 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOL_7.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_7.BackColor = Color.Red;
            }
            else
            {
                s9 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOL_7.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOL_7.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOL_7.Text = s9; }));


        }

        //GPIO Bit Polarity Selection for the LB_IOH Connector
        private void gpioBitSelectionLB_IOH()
        {
            int index = 0;
            int gpioLB_IOH_0 = 0;
            int gpioLB_IOH_1 = 0;
            int gpioLB_IOH_2 = 0;
            int gpioLB_IOH_3 = 0;
            int gpioLB_IOH_4 = 0;
            int gpioLB_IOH_5 = 0;
            int gpioLB_IOH_6 = 0;
            int gpioLB_IOH_7 = 0;


            for (index = 0; index < 8; index++)
            {
                //gpioLB_IOH_0 = ((int)gpioLB_IOH & (1 << index));
                switch (index)
                {
                    case 0:
                        gpioLB_IOH_0 = ((int)gpioLB_IOH & (1 << index));
                        break;
                    case 1:
                        gpioLB_IOH_1 = ((int)gpioLB_IOH & (1 << index));
                        break;
                    case 2:
                        gpioLB_IOH_2 = ((int)gpioLB_IOH & (1 << index));
                        break;
                    case 3:
                        gpioLB_IOH_3 = ((int)gpioLB_IOH & (1 << index));
                        break;
                    case 4:
                        gpioLB_IOH_4 = ((int)gpioLB_IOH & (1 << index));
                        break;
                    case 5:
                        gpioLB_IOH_5 = ((int)gpioLB_IOH & (1 << index));
                        break;
                    case 6:
                        gpioLB_IOH_6 = ((int)gpioLB_IOH & (1 << index));
                        break;
                    case 7:
                        gpioLB_IOH_7 = ((int)gpioLB_IOH & (1 << index));
                        break;
                    default: break;
                }
            }
            //Set the LB_IOH Connector Pins to High, Low, Tri
            string s2 = String.Empty;
            if (gpioLB_IOH_0 == 0x01)
            {
                s2 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_0.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_0.BackColor = Color.Red;
            }
            else
            {
                s2 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_0.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_0.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_0.Text = s2; }));

            string s3 = String.Empty;
            if (gpioLB_IOH_1 == 0x02)
            {
                s3 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_1.BackColor = Color.Red;
            }
            else
            {
                s3 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_1.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_1.Text = s3; }));

            string s4 = String.Empty;
            if (gpioLB_IOH_2 == 0x04)
            {
                s4 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_2.BackColor = Color.Red;
            }
            else
            {
                s4 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_2.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_2.Text = s4; }));

            string s5 = String.Empty;
            if (gpioLB_IOH_3 == 0x08)
            {
                s5 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_3.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_3.BackColor = Color.Red;
            }
            else
            {
                s5 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_3.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_3.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_3.Text = s5; }));

            string s6 = String.Empty;
            if (gpioLB_IOH_4 == 0x10)
            {
                s6 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_4.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_4.BackColor = Color.Red;
            }
            else
            {
                s6 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_4.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_4.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_4.Text = s6; }));

            string s7 = String.Empty;
            if (gpioLB_IOH_5 == 0x20)
            {
                s7 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_5.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_5.BackColor = Color.Red;
            }
            else
            {
                s7 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_5.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_5.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_5.Text = s7; }));

            string s8 = String.Empty;
            if (gpioLB_IOH_6 == 0x40)
            {
                s8 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_6.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_6.BackColor = Color.Red;
            }
            else
            {
                s8 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_6.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_6.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_6.Text = s8; }));

            string s9 = String.Empty;
            if (gpioLB_IOH_7 == 0x80)
            {
                s9 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_7.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_7.BackColor = Color.Red;
            }
            else
            {
                s9 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_7.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_7.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_7.Text = s9; }));


        }


        //GPIO Bit Polarity Selection for the LB_AD Connector
        private void gpioBitSelectionLB_AD()
        {
            int index = 0;
            int gpioLB_AD_0 = 0;
            int gpioLB_AD_1 = 0;
            int gpioLB_AD_2 = 0;
            int gpioLB_AD_3 = 0;
            int gpioLB_AD_4 = 0;
            int gpioLB_AD_5 = 0;
            int gpioLB_AD_6 = 0;
            int gpioLB_AD_7 = 0;


            for (index = 0; index < 8; index++)
            {
                //gpioLB_AD_0 = ((int)gpioLB_AD & (1 << index));
                switch (index)
                {
                    case 0:
                        gpioLB_AD_0 = ((int)gpioLB_AD & (1 << index));
                        break;
                    case 1:
                        gpioLB_AD_1 = ((int)gpioLB_AD & (1 << index));
                        break;
                    case 2:
                        gpioLB_AD_2 = ((int)gpioLB_AD & (1 << index));
                        break;
                    case 3:
                        gpioLB_AD_3 = ((int)gpioLB_AD & (1 << index));
                        break;
                    case 4:
                        gpioLB_AD_4 = ((int)gpioLB_AD & (1 << index));
                        break;
                    case 5:
                        gpioLB_AD_5 = ((int)gpioLB_AD & (1 << index));
                        break;
                    case 6:
                        gpioLB_AD_6 = ((int)gpioLB_AD & (1 << index));
                        break;
                    case 7:
                        gpioLB_AD_7 = ((int)gpioLB_AD & (1 << index));
                        break;
                    default: break;
                }
            }
            //Set the LB_AD Connector Pins to High, Low, Tri
            string s2 = String.Empty;
            if (gpioLB_AD_0 == 0x01)
            {
                s2 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_AD_0.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_0.BackColor = Color.Red;
            }
            else
            {
                s2 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_AD_0.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_0.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_AD_0.Text = s2; }));

            string s3 = String.Empty;
            if (gpioLB_AD_1 == 0x02)
            {
                s3 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_AD_1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_1.BackColor = Color.Red;
            }
            else
            {
                s3 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_AD_1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_1.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_AD_1.Text = s3; }));

            string s4 = String.Empty;
            if (gpioLB_AD_2 == 0x04)
            {
                s4 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_AD_2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_2.BackColor = Color.Red;
            }
            else
            {
                s4 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_AD_2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_2.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_AD_2.Text = s4; }));

            string s5 = String.Empty;
            if (gpioLB_AD_3 == 0x08)
            {
                s5 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_AD_3.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_3.BackColor = Color.Red;
            }
            else
            {
                s5 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_AD_3.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_3.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_AD_3.Text = s5; }));

            string s6 = String.Empty;
            if (gpioLB_AD_4 == 0x10)
            {
                s6 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_AD_4.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_4.BackColor = Color.Red;
            }
            else
            {
                s6 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_AD_4.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_4.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_AD_4.Text = s6; }));

            string s7 = String.Empty;
            if (gpioLB_AD_5 == 0x20)
            {
                s7 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_AD_5.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_5.BackColor = Color.Red;
            }
            else
            {
                s7 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_AD_5.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_5.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_AD_5.Text = s7; }));

/*            string s8 = String.Empty;
            if (gpioLB_AD_6 == 0x40)
            {
                s8 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_AD_6.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_6.BackColor = Color.Red;
            }
            else
            {
                s8 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_AD_6.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_6.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_AD_6.Text = s8; }));

            string s9 = String.Empty;
            if (gpioLB_AD_7 == 0x80)
            {
                s9 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_AD_7.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_7.BackColor = Color.Red;
            }
            else
            {
                s9 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_AD_7.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_AD_7.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_AD_7.Text = s9; }));

*/
        }

        //GPIO Bit Polarity Selection for the LB_SER Connector
        private void gpioBitSelectionLB_SER()
        {
            int index = 0;
            int gpioLB_SER_0 = 0;
            int gpioLB_SER_1 = 0;
            int gpioLB_SER_2 = 0;
            int gpioLB_SER_3 = 0;
            int gpioLB_SER_4 = 0;
            int gpioLB_SER_5 = 0;
            int gpioLB_SER_6 = 0;
            int gpioLB_SER_7 = 0;


            for (index = 0; index < 8; index++)
            {
                //gpioLB_SER_0 = ((int)gpioLB_SER & (1 << index));
                switch (index)
                {
                    case 0:
                        gpioLB_SER_0 = ((int)gpioLB_SER & (1 << index));
                        break;
                    case 1:
                        gpioLB_SER_1 = ((int)gpioLB_SER & (1 << index));
                        break;
                    case 2:
                        gpioLB_SER_2 = ((int)gpioLB_SER & (1 << index));
                        break;
                    case 3:
                        gpioLB_SER_3 = ((int)gpioLB_SER & (1 << index));
                        break;
                    case 4:
                        gpioLB_SER_4 = ((int)gpioLB_SER & (1 << index));
                        break;
                    case 5:
                        gpioLB_SER_5 = ((int)gpioLB_SER & (1 << index));
                        break;
                    case 6:
                        gpioLB_SER_6 = ((int)gpioLB_SER & (1 << index));
                        break;
                    case 7:
                        gpioLB_SER_7 = ((int)gpioLB_SER & (1 << index));
                        break;
                    default: break;
                }
            }
            //Set the LB_SER Connector Pins to High, Low, Tri
            string s2 = String.Empty;
            if (gpioLB_SER_0 == 0x01)
            {
                s2 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_SER_0.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_SER_0.BackColor = Color.Red;
            }
            else
            {
                s2 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_SER_0.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_SER_0.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_SER_0.Text = s2; }));

            string s3 = String.Empty;
            if (gpioLB_SER_1 == 0x02)
            {
                s3 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_SER_1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_SER_1.BackColor = Color.Red;
            }
            else
            {
                s3 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_SER_1.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_SER_1.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_SER_1.Text = s3; }));

/*            string s4 = String.Empty;
            if (gpioLB_IOH_2 == 0x04)
            {
                s4 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_2.BackColor = Color.Red;
            }
            else
            {
                s4 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_2.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_2.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_2.Text = s4; }));

            string s5 = String.Empty;
            if (gpioLB_IOH_3 == 0x08)
            {
                s5 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_3.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_3.BackColor = Color.Red;
            }
            else
            {
                s5 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_3.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_3.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_3.Text = s5; }));

            string s6 = String.Empty;
            if (gpioLB_IOH_4 == 0x10)
            {
                s6 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_4.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_4.BackColor = Color.Red;
            }
            else
            {
                s6 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_4.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_4.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_4.Text = s6; }));

            string s7 = String.Empty;
            if (gpioLB_IOH_5 == 0x20)
            {
                s7 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_5.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_5.BackColor = Color.Red;
            }
            else
            {
                s7 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_5.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_5.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_5.Text = s7; }));

            string s8 = String.Empty;
            if (gpioLB_IOH_6 == 0x40)
            {
                s8 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_6.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_6.BackColor = Color.Red;
            }
            else
            {
                s8 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_6.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_6.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_6.Text = s8; }));

            string s9 = String.Empty;
            if (gpioLB_IOH_7 == 0x80)
            {
                s9 = char.ConvertFromUtf32(0x2191);
                btnGPIO_LB_IOH_7.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_7.BackColor = Color.Red;
            }
            else
            {
                s9 = char.ConvertFromUtf32(0x2193);
                btnGPIO_LB_IOH_7.ForeColor = System.Drawing.Color.FromArgb(255, 255, 255);
                btnGPIO_LB_IOH_7.BackColor = Color.DodgerBlue;
            }

            this.Invoke(new MethodInvoker(delegate () { btnGPIO_LB_IOH_7.Text = s9; }));
*/

        }


        private void btnLED1Green_Click(object sender, EventArgs e)
        {

            led1GreenControl();
        }
        
        public void led1GreenControl()
        {
            byte ledData = 0x00;

            if (led1GreenEnable)
            {
                ledData = 0x01;
                led1GreenEnable = false;
                btnLED1Green.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led1GreenEnable = true;
                btnLED1Green.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)1);
        }

        private void btnLED1Red_Click(object sender, EventArgs e)
        {
            led1RedControl();
        }
        public void led1RedControl()
        {

            byte ledData = 0x00;

            if (led1RedEnable)
            {
                ledData = 0x02;
                led1RedEnable = false;
                btnLED1Red.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led1RedEnable = true;
                btnLED1Red.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)1);

        }

        private void btnLED1Blue_Click(object sender, EventArgs e)
        {
            led1BlueControl();
        }
        public void led1BlueControl()
        {

            byte ledData = 0x00;

            if (led1BlueEnable)
            {
                ledData = 0x04;
                led1BlueEnable = false;
                btnLED1Blue.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led1BlueEnable = true;
                btnLED1Blue.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)1);

        }

        private void btnLED2Green_Click(object sender, EventArgs e)
        {
            led2GreenControl();
        }

        public void led2GreenControl()
        {
            byte ledData = 0x00;

            if (led2GreenEnable)
            {
                ledData = 0x01;
                led2GreenEnable = false;
                btnLED2Green.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led2GreenEnable = true;
                btnLED2Green.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)2);
        }

        private void btnLED2Red_Click(object sender, EventArgs e)
        {
            led2RedControl();
        }
        public void led2RedControl()
        {

            byte ledData = 0x00;

            if (led2RedEnable)
            {
                ledData = 0x02;
                led2RedEnable = false;
                btnLED2Red.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led2RedEnable = true;
                btnLED2Red.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)2);


        }

        private void btnLED2Blue_Click(object sender, EventArgs e)
        {
            led2BlueControl();
        }
        public void led2BlueControl()
        {

            byte ledData = 0x00;

            if (led2BlueEnable)
            {
                ledData = 0x04;
                led2BlueEnable = false;
                btnLED2Blue.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led2BlueEnable = true;
                btnLED2Blue.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)2);


        }

        private void btnLED3Green_Click(object sender, EventArgs e)
        {
            led3GreenControl();
        }

        public void led3GreenControl()
        {
            byte ledData = 0x00;

            if (led3GreenEnable)
            {
                ledData = 0x01;
                led3GreenEnable = false;
                btnLED3Green.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led3GreenEnable = true;
                btnLED3Green.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)4);

        }

        private void btnLED3Red_Click(object sender, EventArgs e)
        {
            led3RedControl();
        }
        public void led3RedControl()
        {

            byte ledData = 0x00;

            if (led3RedEnable)
            {
                ledData = 0x02;
                led3RedEnable = false;
                btnLED3Red.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led3RedEnable = true;
                btnLED3Red.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)4);



        }

        private void btnLED3Blue_Click(object sender, EventArgs e)
        {
            led3BlueControl();
        }
        public void led3BlueControl()
        {

            byte ledData = 0x00;

            if (led3BlueEnable)
            {
                ledData = 0x04;
                led3BlueEnable = false;
                btnLED3Blue.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led3BlueEnable = true;
                btnLED3Blue.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)4);



        }

        private void btnLED4Green_Click(object sender, EventArgs e)
        {
            led4GreenControl();
        }

        public void led4GreenControl()
        {
            byte ledData = 0x00;

            if (led4GreenEnable)
            {
                ledData = 0x01;
                led4GreenEnable = false;
                btnLED4Green.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led4GreenEnable = true;
                btnLED4Green.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)8);


        }

        private void btnLED4Red_Click(object sender, EventArgs e)
        {

            led4RedControl();
        }
        public void led4RedControl()
        {

            byte ledData = 0x00;

            if (led4RedEnable)
            {
                ledData = 0x02;
                led4RedEnable = false;
                btnLED4Red.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led4RedEnable = true;
                btnLED4Red.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)8);



        }

        private void btnLED4Blue_Click(object sender, EventArgs e)
        {
            led4BlueControl();
        }
        public void led4BlueControl()
        {

            byte ledData = 0x00;

            if (led4BlueEnable)
            {
                ledData = 0x04;
                led4BlueEnable = false;
                btnLED4Blue.Text = "Off";
            }
            else
            {
                ledData = 0x00;
                led4BlueEnable = true;
                btnLED4Blue.Text = "On";
            }

            EPT_AH_SendByte((byte)ledByteAddress, (byte)ledData);

            Thread.Sleep(10);

            EPT_AH_SendTrigger((byte)8);




        }

        private void btnGPIOReadAll_Click(object sender, EventArgs e)
        {
            gpioReadAllPins();
        }


        public void gpioReadAllPins()
        {


            Thread.Sleep(10);

            if (gpioSingleLoopFlag)
            {

                EPT_AH_SendTrigger((byte)255);
            }
            else if(gpioInfiniteLoopFlag & !gpioInfiniteLoopEnabled)
            {
                btnGPIOReadAll.Text = "Stop";
                gpioInfiniteLoopEnabled = true;
                //Set timerDevice to Enabled to start the infinite loop
                if (!timerDevice.Enabled)
                    timerDevice.Enabled = true;

            }
            else if (gpioInfiniteLoopFlag & gpioInfiniteLoopEnabled)
            {
                btnGPIOReadAll.Text = "Start";
                gpioInfiniteLoopEnabled = false;
                //Set timerDevice to dis-abled, exit the inifite loop
                if (timerDevice.Enabled)
                    timerDevice.Enabled = false;

            }



        }

        //Timer1 tick maintains the global counter for Hyper Serial Port
        private void timerDevice_Tick(object sender, EventArgs e)
        {
            if (serialPort_AH.IsOpen)
            {
                gpioInifiniteLoopExecute();

            }
        }

        private void gpioInifiniteLoopExecute()
        {
            EPT_AH_SendTrigger((byte)255);

        }

            private void btnGPIO_LB_XIOLA_0_Click(object sender, EventArgs e)
        {

        }

/*        public void gpioLBXIOLA_0()
        {
            btnGPIO_LB_XIOLA_0.Text = char.ConvertFromUtf32(0x2193);
            //Button2.Text = char.ConvertFromUtf32(0x2191);

        }
*/
        private void rbGPIOInfiniteLoop_CheckedChanged(object sender, EventArgs e)
        {

            gpioInfiniteLoopFlag = true;
            gpioSingleLoopFlag = false;
            gpioInfiniteLoopEnabled = false;
        }

        private void rbGPIOSingleLoop_CheckedChanged(object sender, EventArgs e)
        {

            gpioInfiniteLoopFlag = false;
            gpioSingleLoopFlag = true;
            gpioInfiniteLoopEnabled = false;
        }

        private void btnSlideSwRead_Click(object sender, EventArgs e)
        {
            slideSwitchRead();
        }

        public void slideSwitchRead()
        {
            //Send the Trigger bit 0x10 to the Mega
            EPT_AH_SendTrigger((byte)16);

        }
    }

    // Transfer object definition
    public class Transfer
    {
        public int Command;
        public int Address;
        public int Length;
        public int Payload;
        public byte[] cBlockBuf;
        public bool TransferPending;
        public uint Repititions;

        public Transfer()
        {
            Command = 0;
            Address = 0;
            Length = 0;
            Payload = 0;
            TransferPending = false;
            Repititions = 0;
        }

    }

}

